@@ -1,68 +1,69 @@
 #' Recreate R Library
 #'
 #' These functions can be used to recreate an existing library on a new installation of \R.
-#' The \code{SavePackageDetails} function writes the details of installed packages to a file.
+#' The \code{SavePackageDetails} function writes the details of installed packages to a text file.
 #' And the \code{RecreateLibrary} function reads this file and downloads and installs any
 #' \sQuote{missing} packages from the Comprehensive R Archive Network (CRAN),
-#' CRAN-like and local repositories, and GitHub.
+#' CRAN-like repositories, local package-installation files, and GitHub.
 #'
 #' @param file 'character'.
 #'   Name of the file for reading (or writing) the list of package details.
 #'   For a file name that does not contain an absolute path,
-#'   the name is assumed relative to the current working directory [see \code{\link{getwd}()} command].
+#'   the name is assumed relative to the current working directory (see \code{\link{getwd}} function).
 #'   A \file{.gz} file extension indicates the file is compressed by \emph{gzip}.
 #' @param lib 'character'.
-#'   The library tree(s) to search through when locating installed packages (see \code{\link{.libPaths}}),
+#'   The library tree(s) to search through when locating installed packages (see \code{\link{.libPaths}} function),
 #'   or the library directory where to install packages.
 #' @param repos 'character'.
-#'   Vector of base URL(s) of the CRAN-like repositories (and including CRAN) to use when installing packages.
+#'   Vector of base URL(s) of the CRAN-like repositories (includes CRAN) to use when installing packages.
 #'   For example, the URL of the RStudio sponsored CRAN mirror is \code{"https://cloud.r-project.org/"}.
-#'   And the URL of the Geological Survey R Archive Network (GRAN) is \code{"https://owi.usgs.gov/R"}.
-#' @param snapshot 'logical', 'character', or 'Date'.
+#'   And the URL of the Geological Survey R Archive Network (\href{https://owi.usgs.gov/R/gran.html}{GRAN})
+#'   is \code{"https://owi.usgs.gov/R"}.
+#' @param snapshot 'logical', 'Date', or 'character'.
 #'   Calendar date for a CRAN snapshot in time,
 #'   see the Microsoft R Application Network
-#'   (\href{https://mran.microsoft.com/timemachine/}{MRAN}) website for details.
-#'   If true, the snapshot date is read from the first line of \code{file}.
-#'   A snapshot date can also be specified directly using the required date format, \code{"\%Y-\%m-\%d"}.
-#'   This argument masks all CRAN mirrors in \code{repos}.
+#'   (\href{https://mran.microsoft.com/}{MRAN}) website for details.
+#'   If true, the snapshot date is read from the first line of the package-details \code{file}.
+#'   A snapshot date can also be specified directly using the required date format, YYYY-MM-DD.
+#'   This argument masks any CRAN mirror specified in \code{repos}.
 #' @param local 'character'.
 #'   Vector of paths to local repositories.
 #'   Packages are installed from local files in these directories.
 #'   Files can contain \emph{binary} builds of packages (\file{.zip} on Windows and \file{.tgz} on macOS)
 #'   or be \emph{source} packages (\file{.tar.gz}).
 #' @param versions 'logical'.
-#'   If true, installed package versions will be identical to version numbers stored in \code{file}.
-#'   Only applies to packages from CRAN-like repositories and local files.
+#'   If true, installed package versions will be identical to version numbers stored in the package-details \code{file}.
+#'   Only applies to packages from CRAN-like repositories and local package-installation files.
 #'   Requires that the \pkg{devtools} package is available,
 #'   see \code{\link[devtools]{install_version}} function.
 #' @param github 'logical'.
-#'   If true, an attempt is made to install a subset of packages from GitHub repositories.
+#'   If true, an attempt is made to install a subset of packages from \href{https://github.com/}{GitHub}.
 #'   Only applies to packages missing from the CRAN-like repositories (see \code{repos} argument).
 #'   Requires that the \pkg{githubinstall} package is available,
 #'   see \code{\link[githubinstall]{gh_install_packages}} function.
 #'   Note that locating \R packages hosted on GitHub using nothing but the package name can be difficult.
-#'   The user will be prompted with suggested repository names to identify the correct package to install.
+#'   Therefore, the user may be prompted with suggested repository names to identify the correct package to install.
+#'   Package vignettes are not built using this option.
 #'   An example of an \R package that is only available on GitHub is \pkg{AnomalyDetection},
 #'   located at \href{https://github.com/twitter/AnomalyDetection}{twitter/AnomalyDetection}.
-#'   Package vignettes are not built using this option.
 #' @param quiet 'logical'.
 #'   If true, reduce the amount of output.
 #' @param pkg 'character'.
 #'   One or more names of packages located under \code{lib}.
-#'   Only packages in \code{pkg}, and the packages that \code{pkg} depend on/link to/import/suggest,
-#'   are included in the package details \code{file}.
+#'   Only packages specified in \code{pkg}, and the packages that \code{pkg} depend on/link to/import/suggest,
+#'   will be included in the package-details \code{file}.
 #'
 #' @details A typical workflow is as follows:
 #'   Run the \code{SavePackageDetails()} command on an older version of \R.
 #'   It will print to a text file a complete list of details for packages located under your current \R library tree(s).
-#'   Uninstall the older version of \R if no longer needed.
+#'   If the older version of \R no longer needed, uninstall it.
 #'   Then, on a freshly installed version of \R with the \pkg{inlmisc} package available,
 #'   run the \code{RecreateLibrary()} command.
-#'   It will download and install the packages listed in the package description \code{file}.
+#'   It will download and install the packages listed in the package-details \code{file}.
 #'
 #'   The type of package to download and install from CRAN-like repositories is
 #'   \emph{binary} on Windows and some macOS builds, and \emph{source} on all others.
-#'   Package installation from a GitHub repository or local \file{.tar.gz} file is always a source installation.
+#'   Package installation from GitHub or a local \file{.tar.gz} file is always a source installation.
 #'   If a package is installed from source, and it contains code that needs compiling,
 #'   you must have a working development environment.
 #'   On Windows, install the \href{https://cran.r-project.org/bin/windows/Rtools/}{Rtools} collection
@@ -77,32 +78,31 @@
 #'   install the \R version that was available from CRAN on the
 #'   \href{https://mran.microsoft.com/snapshot/}{snapshot date}.
 #'
-#'   The package details \code{file} is of the following format:
-#'
-#'   # Date modified: YYYY-MM-DD HH:MM:SS UTC \cr
-#'   # R version 9.9.9 (YYYY-MM-DD)
-#'   \tabular{ll}{
-#'   Package \tab Version \cr
-#'   name \tab 9.9.9 \cr
-#'   ... \tab ...
+#'   The package-details \code{file} is of the following format:
+#'   \preformatted{
+#'     # Date modified: YYYY-MM-DD HH:MM:SS UTC
+#'     # R version 9.9.9 (YYYY-MM-DD)
+#'     Package Version
+#'     name    9.9.9
+#'     ...     ...
 #'   }
 #'
-#'   The format is flexible enough to add additional extraneous metadata and table fields.
-#'   For example,
-#'
-#'   # Date modified: 2017-08-12 05:14:33 UTC \cr
-#'   # R version 3.4.1 (2017-06-30) \cr
-#'   # Running under: Windows 10 x64 (build 14393) \cr
-#'   # Platform: x86_64-w64-mingw32
-#'   \tabular{lllll}{
-#'   Package \tab Version \tab Priority \tab Depends \tab Imports \cr
-#'   akima \tab 0.6-2 \tab NA \tab R (>= 2.0.0) \tab sp \cr
-#'   animation \tab 2.5 \tab NA \tab R (>= 2.14.0) \tab NA
+#'   Where the first two lines are reserved for the timestamp and \R-version number, respectively.
+#'   And package data are stored in a tabular structure, that is, data-table values are separated by a \kbd{TAB} character.
+#'   The data format is flexible enough to add additional extraneous metadata and table fields, for example,
+#'   \preformatted{
+#'     # Date modified: 2017-08-12 05:14:33 UTC
+#'     # R version 3.4.1 (2017-06-30)
+#'     # Running under: Windows 10 x64 (build 14393)
+#'     # Platform: x86_64-w64-mingw32
+#'     Package   Version Priority Depends       Imports
+#'     akima     0.6-2   NA       R (>= 2.0.0)  sp
+#'     animation 2.5     NA       R (>= 2.14.0) NA
 #'   }
 #'
-#' @return The \code{SavePackageDetails} function returns (invisibly) the MD5 checksum of the \code{file} content.
-#'   Any changes in the file content will produce a different MD5 checksum.
-#'   Use the \code{\link[tools]{md5sum}} function to verify that the file has not been tampered with.
+#' @return The \code{SavePackageDetails} function returns (invisibly) the MD5 hash of the package-details \code{file} content.
+#'   Any changes in the file content will produce a different MD5 hash.
+#'   Use the \code{\link[tools]{md5sum}} function to verify that a file has not been changed.
 #'   The \code{RecreateLibrary} function returns (invisibly) \code{NULL}.
 #'
 #' @note This package-installation method does not offer one-hundred percent reproducibility of existing \R libraries.
@@ -111,9 +111,9 @@
 #'   both of which provide robust tools for dependency management in \R.
 #'
 #'   If affiliated with the U.S. Department of Interior (DOI), you may receive the following error message:
-#'   "SSL certificate problem: unable to get local issuer certificate".
+#'   \dQuote{SSL certificate problem: unable to get local issuer certificate}.
 #'   The error results from a missing X.509 certificate that permits the DOI to scan encrypted data for security reasons.
-#'   A fix for this error is provided by the \code{\link{AddCertificate}} function.
+#'   A fix for this error is given in the \code{\link{AddCertificate}} function documentation.
 #'
 #' @author J.C. Fisher, U.S. Geological Survey, Idaho Water Science Center
 #'
@@ -128,7 +128,7 @@
 #' \dontrun{
 #' # Run on new version of R, and ensure 'inlmisc' package is available.
 #' repos <- c(CRAN = "https://cloud.r-project.org/", GRAN = "https://owi.usgs.gov/R")
-#' if (!requireNamespace("inlmisc", quietly = TRUE))
+#' if (system.file(package = "inlmisc") == "")
 #'   utils::install.packages("inlmisc", repos = repos["CRAN"], dependencies = TRUE)
 #' inlmisc::RecreateLibrary(repos = repos, github = TRUE)
 #' }
@@ -173,7 +173,8 @@ RecreateLibrary <- function(file="R-packages.tsv", lib=.libPaths()[1],
     r_ver_new <- meta[is]
     r_ver_old <- R.version$version.string
     if (!identical(r_ver_old, r_ver_new)) {
-      fmt <- paste("Your %s is different from the R version read from the file.",
+      fmt <- paste("Current %s version is different from the version used",
+                   "when creating the package-details file.",
                    "If compatiblity is an issue, consider installing %s.")
       msg <- sprintf(fmt, r_ver_new, r_ver_old)
       message(paste(strwrap(msg), collapse="\n"))
@@ -311,9 +312,6 @@ SavePackageDetails <- function(file="R-packages.tsv", lib=.libPaths(), pkg=NULL)
   # get names of all packages under library tree(s)
   pkgs <- utils::installed.packages(lib, noCache=TRUE)
 
-  # remove library-path column
-  pkgs <- pkgs[, colnames(pkgs) != "LibPath"]
-
   # remove newlines from table elements
   pkgs <- apply(pkgs, 2, function(i) gsub("[\r\n]", "", i))
 
@@ -358,8 +356,7 @@ SavePackageDetails <- function(file="R-packages.tsv", lib=.libPaths(), pkg=NULL)
   meta <- c(sprintf("# Date modified: %s UTC", format(Sys.time(), tz="GMT")),
             sprintf("# %s", R.version$version.string),
             sprintf("# Running under: %s", utils::sessionInfo()$running),
-            sprintf("# Platform: %s", R.version$platform),
-            "")
+            sprintf("# Platform: %s", R.version$platform))
   writeLines(meta, con)
 
   # write package details to file
@@ -367,14 +364,14 @@ SavePackageDetails <- function(file="R-packages.tsv", lib=.libPaths(), pkg=NULL)
   suppressWarnings(utils::write.table(pkgs, con, append=TRUE, quote=FALSE,
                                       sep="\t", row.names=FALSE))
 
-  # write file path and md5 checksum to console
+  # write file path and md5 hash to console
   msg <- sprintf("File path:\n %s", normalizePath(path.expand(file)))
   message(msg)
   md5 <- tools::md5sum(file)
-  msg <- sprintf("MD5 checksum:\n %s", md5)
+  msg <- sprintf("MD5 hash:\n %s", md5)
   message(msg)
 
-  # return md5 checksum
+  # return md5 hash
   invisible(md5)
 }
 
@@ -385,7 +382,7 @@ SavePackageDetails <- function(file="R-packages.tsv", lib=.libPaths(), pkg=NULL)
 #' @param lib 'character'.
 #'   Vector of library tree(s)
 #'
-#' @return A 'logical' vector the length of 'x'
+#' @return A 'logical' vector the length of \code{x}.
 #'
 
 .IsPackageInstalled <- function(x, lib) {
